from ilivalidator import Ilivalidator
import os
import tempfile

TEST_DATA_PATH = "ilivalidator/tests/data/"

def test_validate_file_ok():
    valid = Ilivalidator.validate([TEST_DATA_PATH+"OeREBKRM_V2_0_Gesetze.xml"])
    assert valid == True

def test_validate_multiple_files_ok():
    valid = Ilivalidator.validate([TEST_DATA_PATH+"/OeREBKRM_V2_0_Gesetze.xml",TEST_DATA_PATH+"OeREBKRM_V2_0_Themen.xml"])
    assert valid == True

def test_validate_multiple_files_allobjectsaccessible_on_ok():
    settings = {Ilivalidator.SETTING_ALL_OBJECTS_ACCESSIBLE: True}
    valid = Ilivalidator.validate([TEST_DATA_PATH+"OeREBKRM_V2_0_Gesetze.xml",TEST_DATA_PATH+"OeREBKRM_V2_0_Themen.xml"], settings)
    assert valid == True

def test_validate_multiple_files_allobjectsaccessible_on_fail():
    settings = {Ilivalidator.SETTING_ALL_OBJECTS_ACCESSIBLE: True}
    valid = Ilivalidator.validate([TEST_DATA_PATH+"OeREBKRM_V2_0_Gesetze.xml",TEST_DATA_PATH+"OeREBKRM_V2_0_Themen_missing_REF.xml"], settings)
    assert valid == False

def test_validate_multiple_files_allobjectsaccessible_off_ok():
    settings = {Ilivalidator.SETTING_ALL_OBJECTS_ACCESSIBLE: False}
    valid = Ilivalidator.validate([TEST_DATA_PATH+"OeREBKRM_V2_0_Gesetze.xml",TEST_DATA_PATH+"OeREBKRM_V2_0_Themen_missing_REF.xml"], settings)
    assert valid == True

def test_validate_logfiles_on_ok():
    temp_dir = tempfile.TemporaryDirectory()
    log_file = os.path.join(temp_dir.name, "mylog.log")
    xtf_log_file = log_file + ".xtf"

    settings = {Ilivalidator.SETTING_LOGFILE: log_file, Ilivalidator.SETTING_LOGFILE_TIMESTAMP: True, Ilivalidator.SETTING_XTFLOG: xtf_log_file}
    valid = Ilivalidator.validate([TEST_DATA_PATH+"OeREBKRM_V2_0_Gesetze.xml"], settings)
    assert valid == True

    with open(log_file, 'r') as file:
        content = file.read()
        assert -1 != content.find("Info: ...validation done")

    with open(xtf_log_file, 'r') as file:
        content = file.read()
        assert -1 != content.find("<Message>...validation done</Message>")


import platform
import json

from ctypes import *
from importlib_resources import files

if platform.uname()[0] == "Windows":
    lib_name = "libilivalidator.dll"
elif platform.uname()[0] == "Linux":
    lib_name = "libilivalidator.so"
else:
    lib_name = "libilivalidator.dylib"

class Ilivalidator:
    SETTING_ILIDIRS = "org.interlis2.validator.ilidirs"
    SETTING_MODELNAMES = "org.interlis2.validator.modelNames"
    SETTING_ALL_OBJECTS_ACCESSIBLE = "org.interlis2.validator.allobjectsaccessible"
    SETTING_LOGFILE = "org.interlis2.validator.log"
    SETTING_LOGFILE_TIMESTAMP = "org.interlis2.validator.log.timestamp"
    SETTING_XTFLOG = "org.interlis2.validator.xtflog"

    @staticmethod
    def validate(data_file_names: list, settings: dict=None) -> bool:
        lib_path = files('ilivalidator.lib_ext').joinpath(lib_name)
        # str() seems to be necessary on windows: https://github.com/TimDettmers/bitsandbytes/issues/30
        dll = CDLL(str(lib_path))
        isolate = c_void_p()
        isolatethread = c_void_p()
        dll.graal_create_isolate(None, byref(isolate), byref(isolatethread))
        dll.ilivalidator.restype = bool

        data_file_names_string = ';'.join(data_file_names)

        if settings is None:
            settings = {}       
        settings_string = json.dumps(settings)

        result = dll.ilivalidator(isolatethread, c_char_p(bytes(data_file_names_string, "utf8")), c_char_p(bytes(settings_string, "utf8")))
        return result
